using AutoMapper;
using TrainingSvc.DTOs;
using TrainingSvc.Entities;
using TrainingSvc.Repositories;
using TrainingSvc.IServices;

namespace TrainingSvc.Services;

public class SessionService : ISessionService
{
    private readonly ISessionRepository _sessionRepo;
    private readonly IExerciceInstanceRepository _exerciceRepo;
    private readonly IExerciceTemplateRepository _exerciceTemplateRepo;
    private readonly IMapper _mapper;

    public SessionService(
        ISessionRepository sessionRepo,
        IExerciceInstanceRepository exerciceRepo,
        IExerciceTemplateRepository exerciceTemplateRepo,
        IMapper mapper)
    {
        _sessionRepo = sessionRepo;
        _exerciceRepo = exerciceRepo;
        _exerciceTemplateRepo = exerciceTemplateRepo;
        _mapper = mapper;
    }

    public async Task<IEnumerable<SessionDto>> GetAllAsync()
    {
        var sessions = await ((SessionRepository)_sessionRepo).GetAllWithExercicesAndTemplatesAsync();
        return _mapper.Map<IEnumerable<SessionDto>>(sessions);
    }

    public async Task<SessionDto?> GetByIdAsync(string id)
    {
        var entity = await ((SessionRepository)_sessionRepo).GetByIdWithExercicesAndTemplatesAsync(id);
        return entity == null ? null : _mapper.Map<SessionDto>(entity);
    }

    public async Task<SessionDto> CreateAsync(CreateSessionDto dto)
    {
        // Ne mappe pas les exercices ici
        var session = _mapper.Map<Session>(dto);
        session.Exercices.Clear(); // S'assure qu'il n'y a pas d'exercices liés

        await _sessionRepo.InsertAsync(session);
        await _sessionRepo.SaveChangesAsync();

        foreach (var exoDto in dto.Exercices)
        {
            if (!string.IsNullOrEmpty(exoDto.ExerciceTemplateId))
            {
                var exists = await _exerciceTemplateRepo.ExistsAsync(t => t.Id == exoDto.ExerciceTemplateId);
                if (!exists)
                    throw new ArgumentException("ExerciceTemplateId invalide.");
            }
            var exo = _mapper.Map<ExerciceInstance>(exoDto);
            exo.SessionId = session.Id;
            await _exerciceRepo.InsertAsync(exo);
        }
        await _exerciceRepo.SaveChangesAsync();

        var entity = await ((SessionRepository)_sessionRepo).GetByIdWithExercicesAndTemplatesAsync(session.Id);
        return _mapper.Map<SessionDto>(entity);
    }

    public async Task<bool> UpdateAsync(string id, UpdateSessionDto dto)
    {
        var session = await ((SessionRepository)_sessionRepo).GetByIdWithExercicesAndTemplatesAsync(id);
        if (session == null) return false;

        _mapper.Map(dto, session);
        _sessionRepo.Update(session);
        await _sessionRepo.SaveChangesAsync();

        return true;
    }

    public async Task<bool> DeleteAsync(string id)
    {
        var session = await _sessionRepo.GetByIdAsync(id, s => s.Exercices);
        if (session == null) return false;

        foreach (var ex in session.Exercices.ToList())
        {
            _exerciceRepo.Delete(ex.Id);
        }
        await _exerciceRepo.SaveChangesAsync();

        _sessionRepo.Delete(id);
        await _sessionRepo.SaveChangesAsync();

        return true;
    }
}